@@ -10,20 +10,20 @@ use crate::{
     },
     transmission,
 };
-use core::{cmp::max, marker::PhantomData, time::Duration};
+use core::{cmp::max, time::Duration};
 use s2n_quic_core::{
     event::{self, builder::CongestionSource, IntoEvent},
     frame,
     frame::ack::EcnCounts,
     inet::{DatagramInfo, ExplicitCongestionNotification},
     packet::number::{PacketNumber, PacketNumberRange, PacketNumberSpace},
-    recovery::{CongestionController, RttEstimator, K_GRANULARITY},
+    recovery::{congestion_controller, CongestionController, RttEstimator, K_GRANULARITY},
     time::{timer, Timer, Timestamp},
     transport,
 };
 use smallvec::SmallVec;
 
-type PacketDetails = (PacketNumber, SentPacketInfo);
+type PacketDetails<PacketInfo> = (PacketNumber, SentPacketInfo<PacketInfo>);
 
 #[derive(Debug)]
 pub struct Manager<Config: endpoint::Config> {
@@ -37,7 +37,7 @@ pub struct Manager<Config: endpoint::Config> {
     //= https://www.rfc-editor.org/rfc/rfc9002#section-A.3
     //# An association of packet numbers in a packet number space to information about them.
     //  These are packets that are pending acknowledgement.
-    sent_packets: SentPackets,
+    sent_packets: SentPackets<<<Config::CongestionControllerEndpoint as congestion_controller::Endpoint>::CongestionController as congestion_controller::CongestionController>::PacketInfo>,
 
     // Timer set when packets may be declared lost at a time in the future
     loss_timer: Timer,
@@ -60,8 +60,6 @@ pub struct Manager<Config: endpoint::Config> {
 
     // The total ecn counts for outstanding (unacknowledged) packets
     sent_packet_ecn_counts: EcnCounts,
-
-    config: PhantomData<Config>,
 }
 
 //= https://www.rfc-editor.org/rfc/rfc9002#section-6.1.1
@@ -100,6 +98,17 @@ macro_rules! recovery_event {
     };
 }
 
+// Since `SentPacketInfo` is generic over a type supplied by the Congestion Controller implementation,
+// the type definition is particularly lengthy, especially since rust requires the fully-qualified
+// syntax to eliminate ambiguity. This macro can be used where ever the Congestion Controller
+// generic PacketInfo type is required to help with readability.
+macro_rules! packet_info_type {
+    () => {
+        <<Config::CongestionControllerEndpoint as congestion_controller::Endpoint>::CongestionController as congestion_controller::CongestionController>::PacketInfo
+    }
+}
+
+#[allow(clippy::type_complexity)]
 impl<Config: endpoint::Config> Manager<Config> {
     /// Constructs a new `recovery::Manager`
     pub fn new(space: PacketNumberSpace) -> Self {
@@ -112,7 +121,6 @@ impl<Config: endpoint::Config> Manager<Config> {
             time_of_last_ack_eliciting_packet: None,
             baseline_ecn_counts: EcnCounts::default(),
             sent_packet_ecn_counts: EcnCounts::default(),
-            config: PhantomData,
         }
     }
 
@@ -197,6 +205,14 @@ impl<Config: endpoint::Config> Manager<Config> {
             0
         };
 
+        let path_id = context.path_id();
+        let path = context.path_mut();
+        let cc_packet_info = path.congestion_controller.on_packet_sent(
+            time_sent,
+            congestion_controlled_bytes,
+            &path.rtt_estimator,
+        );
+
         self.sent_packets.insert(
             packet_number,
             SentPacketInfo::new(
@@ -204,13 +220,11 @@ impl<Config: endpoint::Config> Manager<Config> {
                 congestion_controlled_bytes,
                 time_sent,
                 outcome.ack_elicitation,
-                context.path_id(),
+                path_id,
                 ecn,
+                cc_packet_info,
             ),
         );
-
-        let path_id = context.path_id();
-        let path = context.path_mut();
         path.ecn_controller
             .on_packet_sent(ecn, path_event!(path, path_id), publisher);
         self.sent_packet_ecn_counts.increment(ecn);
@@ -225,11 +239,6 @@ impl<Config: endpoint::Config> Manager<Config> {
             let is_handshake_confirmed = context.is_handshake_confirmed();
             let path = context.path_mut_by_id(context.path_id());
             self.update_pto_timer(path, time_sent, is_handshake_confirmed);
-            path.congestion_controller.on_packet_sent(
-                time_sent,
-                congestion_controlled_bytes,
-                &path.rtt_estimator,
-            );
         }
     }
 
@@ -311,8 +320,9 @@ impl<Config: endpoint::Config> Manager<Config> {
     ) -> Result<(), transport::Error> {
         let largest_acked_packet_number =
             self.space.new_packet_number(frame.largest_acknowledged());
-        let mut newly_acked_packets =
-            SmallVec::<[SentPacketInfo; ACKED_PACKETS_INITIAL_CAPACITY]>::new();
+        let mut newly_acked_packets = SmallVec::<
+            [SentPacketInfo<packet_info_type!()>; ACKED_PACKETS_INITIAL_CAPACITY],
+        >::new();
 
         // Update the largest acked packet if the largest packet acked in this frame is larger
         let new_largest_packet = if self
@@ -360,7 +370,7 @@ impl<Config: endpoint::Config> Manager<Config> {
             let (_, largest_newly_acked_info) = largest_newly_acked;
             self.process_new_acked_packets(
                 &newly_acked_packets,
-                largest_newly_acked_info.time_sent,
+                largest_newly_acked_info,
                 new_largest_packet,
                 datagram,
                 &frame,
@@ -383,13 +393,15 @@ impl<Config: endpoint::Config> Manager<Config> {
         Pub: event::ConnectionPublisher,
     >(
         &mut self,
-        newly_acked_packets: &mut SmallVec<[SentPacketInfo; ACKED_PACKETS_INITIAL_CAPACITY]>,
+        newly_acked_packets: &mut SmallVec<
+            [SentPacketInfo<packet_info_type!()>; ACKED_PACKETS_INITIAL_CAPACITY],
+        >,
         datagram: &DatagramInfo,
         frame: &frame::Ack<A>,
         context: &mut Ctx,
         publisher: &mut Pub,
-    ) -> Result<(Option<PacketDetails>, bool), transport::Error> {
-        let mut largest_newly_acked: Option<PacketDetails> = None;
+    ) -> Result<(Option<PacketDetails<packet_info_type!()>>, bool), transport::Error> {
+        let mut largest_newly_acked: Option<PacketDetails<packet_info_type!()>> = None;
         let mut includes_ack_eliciting = false;
 
         for ack_range in frame.ack_ranges() {
@@ -447,7 +459,7 @@ impl<Config: endpoint::Config> Manager<Config> {
 
     fn update_congestion_control<A: frame::ack::AckRanges, Ctx: Context<Config>>(
         &mut self,
-        largest_newly_acked: PacketDetails,
+        largest_newly_acked: PacketDetails<packet_info_type!()>,
         largest_acked_packet_number: PacketNumber,
         includes_ack_eliciting: bool,
         datagram: &DatagramInfo,
@@ -501,8 +513,10 @@ impl<Config: endpoint::Config> Manager<Config> {
         Pub: event::ConnectionPublisher,
     >(
         &mut self,
-        newly_acked_packets: &SmallVec<[SentPacketInfo; ACKED_PACKETS_INITIAL_CAPACITY]>,
-        largest_newly_acked_time_sent: Timestamp,
+        newly_acked_packets: &SmallVec<
+            [SentPacketInfo<packet_info_type!()>; ACKED_PACKETS_INITIAL_CAPACITY],
+        >,
+        largest_newly_acked: SentPacketInfo<packet_info_type!()>,
         new_largest_packet: bool,
         datagram: &DatagramInfo,
         frame: &frame::Ack<A>,
@@ -528,10 +542,11 @@ impl<Config: endpoint::Config> Manager<Config> {
 
             if acked_packet_info.path_id == current_path_id {
                 current_path_acked_bytes += sent_bytes;
-            } else {
-                path.congestion_controller.on_packet_ack(
-                    largest_newly_acked_time_sent,
+            } else if sent_bytes > 0 {
+                path.congestion_controller.on_ack(
+                    acked_packet_info.time_sent,
                     sent_bytes,
+                    acked_packet_info.cc_packet_info,
                     &path.rtt_estimator,
                     datagram.timestamp,
                 );
@@ -580,9 +595,10 @@ impl<Config: endpoint::Config> Manager<Config> {
         let path = context.path_mut();
 
         if current_path_acked_bytes > 0 {
-            path.congestion_controller.on_packet_ack(
-                largest_newly_acked_time_sent,
+            path.congestion_controller.on_ack(
+                largest_newly_acked.time_sent,
                 current_path_acked_bytes,
+                largest_newly_acked.cc_packet_info,
                 &path.rtt_estimator,
                 datagram.timestamp,
             );
@@ -700,7 +716,7 @@ impl<Config: endpoint::Config> Manager<Config> {
         now: Timestamp,
         context: &mut Ctx,
         publisher: &mut Pub,
-    ) -> (Duration, Vec<PacketDetails>) {
+    ) -> (Duration, Vec<PacketDetails<packet_info_type!()>>) {
         let largest_acked_packet = self
             .largest_acked_packet
             .expect("This function is only called after an ack has been received");
@@ -794,17 +810,17 @@ impl<Config: endpoint::Config> Manager<Config> {
         &mut self,
         now: Timestamp,
         persistent_congestion_duration: Duration,
-        sent_packets_to_remove: Vec<PacketDetails>,
+        sent_packets_to_remove: Vec<PacketDetails<packet_info_type!()>>,
         context: &mut Ctx,
         publisher: &mut Pub,
     ) {
         let current_path_id = context.path_id();
         let mut is_congestion_event = false;
+        let mut prev_lost_packet_number = None;
 
         // Remove the lost packets and account for the bytes on the proper congestion controller
         for (packet_number, sent_info) in sent_packets_to_remove {
             let path = context.path_mut_by_id(sent_info.path_id);
-
             self.sent_packets.remove(packet_number);
 
             //= https://www.rfc-editor.org/rfc/rfc9002#section-7.6.2
@@ -817,6 +833,10 @@ impl<Config: endpoint::Config> Manager<Config> {
                 // Check that the packet was sent on this path
                 && sent_info.path_id == current_path_id;
 
+            let new_loss_burst = prev_lost_packet_number.map_or(true, |prev: PacketNumber| {
+                packet_number.checked_distance(prev) != Some(1)
+            });
+
             let mut is_mtu_probe = false;
             if sent_info.sent_bytes as usize > path.mtu_controller.mtu() {
                 //= https://www.rfc-editor.org/rfc/rfc9000#section-14.4
@@ -833,9 +853,11 @@ impl<Config: endpoint::Config> Manager<Config> {
                     .on_packet_discarded(sent_info.sent_bytes as usize);
                 is_mtu_probe = true;
             } else if sent_info.sent_bytes > 0 {
-                path.congestion_controller.on_packets_lost(
+                path.congestion_controller.on_packet_lost(
                     sent_info.sent_bytes as u32,
+                    sent_info.cc_packet_info,
                     persistent_congestion,
+                    new_loss_burst,
                     now,
                 );
                 is_mtu_probe = false;
@@ -878,6 +900,8 @@ impl<Config: endpoint::Config> Manager<Config> {
                 //# persistent congestion is established.
                 path.rtt_estimator.on_persistent_congestion();
             }
+
+            prev_lost_packet_number = Some(packet_number);
         }
 
         if is_congestion_event {
