@@ -1657,11 +1657,27 @@ fn remove_lost_packets_persistent_congestion_path_aware() {
     let sent_packets_to_remove = vec![
         (
             space.new_packet_number(VarInt::from_u8(9)),
-            SentPacketInfo::new(true, 1, now, AckElicitation::Eliciting, first_path_id, ecn),
+            SentPacketInfo::new(
+                true,
+                1,
+                now,
+                AckElicitation::Eliciting,
+                first_path_id,
+                ecn,
+                (),
+            ),
         ),
         (
             space.new_packet_number(VarInt::from_u8(9)),
-            SentPacketInfo::new(true, 1, now, AckElicitation::Eliciting, second_path_id, ecn),
+            SentPacketInfo::new(
+                true,
+                1,
+                now,
+                AckElicitation::Eliciting,
+                second_path_id,
+                ecn,
+                (),
+            ),
         ),
     ];
 
@@ -1925,6 +1941,7 @@ fn persistent_congestion() {
         context.path().congestion_controller.persistent_congestion
     );
     assert_eq!(context.path().rtt_estimator.first_rtt_sample(), None);
+    assert_eq!(1, context.path().congestion_controller.loss_bursts);
 
     // t=20: Send packet #10
     manager.on_packet_sent(
@@ -2078,6 +2095,7 @@ fn persistent_congestion_multiple_periods() {
         Some(true),
         context.path().congestion_controller.persistent_congestion
     );
+    assert_eq!(2, context.path().congestion_controller.loss_bursts);
 }
 
 //= https://www.rfc-editor.org/rfc/rfc9002#section-7.6.2
@@ -2511,6 +2529,7 @@ fn on_timeout() {
             AckElicitation::Eliciting,
             unsafe { path::Id::new(0) },
             ecn,
+            (),
         ),
     );
     manager.pto.timer.set(now - Duration::from_secs(5));
